#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os, time, subprocess, toml
from flask import Flask, flash, request, redirect, url_for
# HTTP auth
from flask_httpauth import HTTPBasicAuth
from werkzeug.security import generate_password_hash, check_password_hash
# FILE UPLOAD
from werkzeug.utils import secure_filename

app = Flask(__name__)

app.config.from_file("defaults.toml", load=toml.load)
config_locations = ["./", "~/.", "~/.config/"]
for location in config_locations:
    # Optional config files, ~ is expanded to $HOME on *nix, %USERPROFILE% on windows
    # ~ app.config.from_file("videopi.toml", load=toml.load, silent=True)
    if app.config.from_file(os.path.expanduser( location + "pilpil-client.toml"), load=toml.load, silent=True):
        print("Found configuration file in " + os.path.expanduser( location ))


#UPLOAD_FOLDER = os.path.expanduser('~/Videos')
#ALLOWED_EXTENSIONS = {'avi', 'mkv', 'mp4'}

UPLOAD_FOLDER = os.path.expanduser(app.config['DEFAULT']['media_folder_local'])
ALLOWED_EXTENSIONS = app.config['DEFAULT']['media_exts']
HTTP_SECRET = str(app.config['DEFAULT']['auth'])
DEBUG = app.config['DEFAULT']['debug']

#HTTPS
#ASSETS_DIR = os.path.dirname(os.path.abspath(__file__))
# HTTP Serve
from waitress import serve

#app.secret_key = b'flafoudi'
app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER
# Max upload size 100M ( see nginx default also )
app.config['MAX_CONTENT_LENGTH'] = 100 * 1000 * 1000

auth = HTTPBasicAuth()
users = {
    "": generate_password_hash(HTTP_SECRET),
}

@auth.verify_password
def verify_password(username, password):
    if username in users and check_password_hash(users.get(username), password):
        return username

signal = 0

# Check file extension is allowed
def allowed_file(filename):
    # Check for dot in filename
    if "." in filename:
        # Split from right at first dot to find ext and allow files with "." in name
        if filename.rsplit(".",1)[-1] in ALLOWED_EXTENSIONS:
            return True

# Get Wifi signal level
def getRSSI():
    signal = subprocess.run( "./get_rssi.sh", capture_output=True)
    print(signal)
    signal = str(signal.stdout, 'UTF-8').strip("-").strip("\n")
    return signal

# Blink the Pi led to allow identification
def blinkPy():
    for j in range(10):
        os.system('echo 1 | sudo dd status=none of=/sys/class/leds/led0/brightness > /dev/null 2>&1') # led on
        time.sleep(.2)
        os.system('echo 0 | sudo dd status=none of=/sys/class/leds/led0/brightness > /dev/null 2>&1') # led off
        time.sleep(.2)
    return "OK"



@app.route("/")
@auth.login_required
def main():
    return "Nothing to see here !"

@app.route("/rssi")
@auth.login_required
def signal():
    return getRSSI()

@app.route("/blink")
@auth.login_required
def blink():
    return blinkPy()

@app.route("/reboot")
@auth.login_required
def reboot():
    stdout = subprocess.run(["sudo", "/usr/sbin/reboot"],  capture_output=True)
    print(stdout)
    return "Rebooting..."

@app.route("/poweroff")
@auth.login_required
def shutdown():
    stdout = subprocess.run(["sudo", "/usr/sbin/poweroff"], capture_output=True)
    print(stdout)
    return "Shuting down..."

# File upload

@app.route('/upload', methods=['GET', 'POST'])
@auth.login_required
def upload_file():
    if request.method == 'POST':
        # check if the post request has the file part
        if 'file' not in request.files:
            return "No file part: " + str(request.files)
        file = request.files['file']
        # If the user does not select a file, the browser submits an
        # empty file without a filename.
        if file.filename == '':
            return 'No selected file'
        if file and allowed_file(file.filename):
            filename = secure_filename(file.filename)
            file.save(os.path.join(UPLOAD_FOLDER, filename))
            return "File saved in " + UPLOAD_FOLDER
    return "OK"

if __name__ == '__main__':
#    app.run()
    serve(app, host='127.0.0.1', port=5000, url_scheme='https')
